import re
import subprocess
import json
from ldap3 import Server, Connection, SAFE_SYNC
from dataEngine import AppConfig, Logger

# LDAP
LDAP_USER = subprocess.getoutput("cat /etc/callweaver/res_ldap.conf | grep 'user=' | cut -d= -f 2,3,4")
LDAP_PASS = subprocess.getoutput("cat /etc/callweaver/res_ldap.conf | grep 'pass=' | cut -d= -f 2")
LDAP_SERVER = Server("localhost")
LDAP_CONN = None

ldap_cache_storage = {}

ATTR_MAP = {
    "uid": "userID",
    "mail": "email",
    "telephoneNumber": "userExtension",
    "userKind": "userKind",
    "cn": "commonName",
    "mobile": "mobile",
    "ou": "department"
}

USER_ATTRIBUTES = list(ATTR_MAP.keys())

logger = Logger.get()


def _get_connection():
    global LDAP_CONN
    if LDAP_CONN is None:
        LDAP_CONN = Connection(LDAP_SERVER, LDAP_USER, LDAP_PASS, client_strategy=SAFE_SYNC, auto_bind=True)
    return LDAP_CONN


def getUserInfoMock(number):
    if AppConfig.get("have_ldap_storage"):
        return ldap_cache_storage.get(number)
    if number and len(number) > 7:
        return {}
    return {"userID": number, "email": "{}@wildix.com".format(number), "userExtension": number, "userKind": "user",
            "commonName": "User {}".format(number), "groupID": "000000{}".format(number), "groupName": "Default",
            "department": "dummyDepartment"}


def getUserInfo(number):
    rv = {}
    if number is None:
        return rv
    if AppConfig.isTest():
        return getUserInfoMock(number)

    if AppConfig.get("cache_ldap") and ldap_cache_storage.get(number) is not None:
        logger.debug("Use LDAP cache for get data for %s" % number)
        return ldap_cache_storage.get(number)

    status, _, response, _ = _get_connection().search(search_base="dc=wildix",
                                                      search_filter="(&(objectClass=wPerson)(|(telephoneNumber=%s)(mobile=%s)))" % (number, number),
                                                      attributes=USER_ATTRIBUTES+["owner"])
    if status and response:
        rv = {ATTR_MAP[attr]:(response[0].get("attributes").get(attr)[0]) for attr in USER_ATTRIBUTES if response[0].get("attributes").get(attr)}
        if "userID" in rv:
            gid = re.findall("^gid=(\d+),.*", response[0].get("attributes").get("owner")[0])
            if gid:
                rv["groupID"] = gid[0]
                rv["groupName"] = _getUserGroupName(gid[0])
    if AppConfig.get("cache_ldap"):
        ldap_cache_storage[number] = rv
    return rv


def _getUserGroupName(gid):
    status, _, response, _ = _get_connection().search(search_base="dc=wildix",
                                              search_filter="(&(objectClass=wGroup)(gid={}))".format(gid),
                                              attributes=["cn"])
    if status and response and "cn" in response[0].get("attributes"):
        return response[0].get("attributes").get("cn")[0]
    return


def store():
    if AppConfig.get("ldap_file"):
        with open(AppConfig.get("ldap_file"), "w") as ldap_file:
            ldap_file.write(json.dumps(ldap_cache_storage))


def read(file_name):
    if AppConfig.isTest():
        with open(file_name, "r") as ldap_file:
            global ldap_cache_storage
            line = ldap_file.readlines()
            ldap_cache_storage = json.JSONDecoder().decode(line[0])
