import logging
import logging.handlers
from dataEngine import AppConfig

logger = logging.getLogger("pbx-data-engine")


def init_logger():
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.DEBUG)
    console_handler.setFormatter(CustomFormatter())

    s_handler = logging.handlers.SysLogHandler(address='/dev/log')
    s_format = logging.Formatter('%(name)s: <%(levelname)s>  %(message)s')
    s_handler.setFormatter(s_format)

    logging.basicConfig(level=logging.DEBUG if AppConfig.get("full_debug") else logging.WARNING,
                        handlers=[console_handler, s_handler])


def get():
    return logger


class CustomFormatter(logging.Formatter):
    grey = "\033[38;1m"
    green = "\033[32;21m"
    yellow = "\033[33;21m"
    red = "\033[31;21m"
    bold_red = "\033[31;1m"
    reset = "\033[0m"
    format=u'[%(asctime)s] [%(levelname)s] [%(filename)s:%(lineno)d] %(message)s'

    FORMATS = {
        logging.DEBUG: green + format + reset,
        logging.INFO: yellow + format + reset,
        logging.WARNING: bold_red + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)